﻿/* SPI Master example

   This example code is in the Public Domain (or CC0 licensed, at your option.)

   Unless required by applicable law or agreed to in writing, this
   software is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR
   CONDITIONS OF ANY KIND, either express or implied.
*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "freertos/FreeRTOS.h"
#include "freertos/task.h"
#include "esp_system.h"

#include "soc/gpio_struct.h"
#include "driver/gpio.h"
#include "spi_master2.h"
#include "test_mp3.h"
#include "esp_log.h"
#include "driver/uart.h"
#include "sd_card.h"

spi_device_handle_t spi;
#define TAG "mp3:"
/*
 This code displays some fancy graphics on the ILI9341-based 320x240 LCD on an ESP-WROVER_KIT board.
 It is not very fast, even when the SPI transfer itself happens at 8MHz and with DMA, because
 the rest of the code is not very optimized. Especially calculating the image line-by-line
 is inefficient; it would be quicker to send an entire screenful at once. This example does, however,
 demonstrate the use of both spi_device_transmit as well as spi_device_queue_trans/spi_device_get_trans_result
 as well as pre-transmit callbacks.

 Some info about the ILI9341: It has an C/D line, which is connected to a GPIO here. It expects this
 line to be low for a command and high for data. We use a pre-transmit callback here to control that
 line: every transaction has as the user-definable argument the needed state of the D/C line and just
 before the transaction is sent, the callback will set this line to the correct state.
*/

#define PIN_NUM_MISO 23
#define PIN_NUM_MOSI 22
#define PIN_NUM_CLK  21
#define PIN_NUM_CS   16

#define PIN_NUM_XRST   19
#define PIN_NUM_DREQ   18
#define PIN_NUM_XDCS   5

uint16_t file_p=0;//文件读取指针

//voice spi read/write
unsigned char voiceSpiReadWrite(spi_device_handle_t spi, const uint8_t cmd)
{
	spi_transaction_t t;
	memset(&t, 0, sizeof(t));
	t.length=8;
	t.tx_buffer=&cmd;               //The data is the cmd itself
	t.flags = SPI_TRANS_USE_RXDATA;
	esp_err_t ret = spi_device_transmit(spi, &t);
	assert( ret == ESP_OK );
	return t.rx_data[0];
}

/******************************************************************
 - 功能描述：向VS1003的功能寄存器中写入数据（一个字，即两个字节）
 - 隶属模块：VS1003B模块
 - 函数属性：外部，用户可调用
 - 参数说明：addr是功能寄存器的地址
             hdat是要写入的高字节
             ldat是要写入的低字节
 - 返回说明：无返回
 ******************************************************************/
void VS_Write_Reg(spi_device_handle_t spi,unsigned char addr,unsigned char hdat,unsigned char ldat)
{
    while(!gpio_get_level(PIN_NUM_DREQ));     //VS1003的DREQ为高电平时才接收数据
    gpio_set_level(PIN_NUM_XDCS, 1);
    gpio_set_level(PIN_NUM_CS,0);            //打开片选，SCI有效，这样才能对功能寄存器进行读写
    voiceSpiReadWrite(spi,VS_Write);//写入操作码0x02   00000010 （功能寄存器写操作）
    voiceSpiReadWrite(spi,addr);  //写入寄存器地址
    voiceSpiReadWrite(spi,hdat);  //写入高字节
    voiceSpiReadWrite(spi,ldat);  //写入低字节
    gpio_set_level(PIN_NUM_CS,1);             //关闭片选，SCI无效
}

/*******************************************************************************
* Function Name  : VS1003_ResetDecodeTime
* Description    : None
* Input          : None
* Output         : None
* Return         : None
* Attention		 : None
*******************************************************************************/
void VS1003_ResetDecodeTime(spi_device_handle_t spi)
{
   VS_Write_Reg(spi,VS_DECODE_TIME,0x00,0x00);
   VS_Write_Reg(spi,VS_DECODE_TIME,0x00,0x00); /* 操作两次 */
}

//Initialize the display
void VS1003_Reset(spi_device_handle_t spi)
{
    gpio_config_t io_conf;
    //Initialize non-SPI GPIOs
    gpio_set_direction(PIN_NUM_XDCS, GPIO_MODE_OUTPUT);
    gpio_set_direction(PIN_NUM_XRST, GPIO_MODE_OUTPUT);
    gpio_set_direction(PIN_NUM_DREQ, GPIO_MODE_INPUT);
    gpio_set_direction(PIN_NUM_CS, GPIO_MODE_OUTPUT);
    //disable interrupt
    io_conf.intr_type = GPIO_PIN_INTR_DISABLE;
	//bit mask of the pins, use GPIO4/5 here
	io_conf.pin_bit_mask = PIN_NUM_DREQ;
	//set as input mode
	io_conf.mode = GPIO_MODE_INPUT;
	//enable pull-up mode
	io_conf.pull_up_en = 1;
	gpio_config(&io_conf);
    //Reset the vs1003
    gpio_set_level(PIN_NUM_XRST, 0);
    vTaskDelay(1000 / portTICK_RATE_MS);
    gpio_set_level(PIN_NUM_XRST, 1);
    gpio_set_level(PIN_NUM_XDCS, 1);//取消数据传输
    gpio_set_level(PIN_NUM_CS, 1);//取消数据传输

    //Send all the commands
    while (gpio_get_level(PIN_NUM_DREQ)==0);
    voiceSpiReadWrite(spi,0xff);
    gpio_set_level(PIN_NUM_XDCS, 1);//取消数据传输
    gpio_set_level(PIN_NUM_CS, 1);//取消数据传输

    VS_Write_Reg(spi,VS_MODE  ,0x08,0x04);  //软件复位，向0号寄存器写入0x0804  SM_SDINEW为1 SM_RESET为1
	VS_Write_Reg(spi,VS_CLOCKF,0x98,0x00);  //时钟设置，向3号寄存器写入0x9800  SC_MULT为4 SC_ADD为3 SC_FREQ为0
	VS_Write_Reg(spi,VS_AUDATA,0x30,0x30);  //设置VS1003的时钟，3倍频，1.5xADD
	VS_Write_Reg(spi,VS_VOL   ,0x30,0x30);  //音量设置，左右声道均最大音量
	VS1003_ResetDecodeTime(spi);     //复位解码时间

	gpio_set_level(PIN_NUM_XDCS, 0);//打开数据片选，注意此时XCS（片选）为高电平，SDI有效
	voiceSpiReadWrite(spi,0);    //写入数据，这里写入4个0，是无关数据，用来启动数据传输
	voiceSpiReadWrite(spi,0);
	voiceSpiReadWrite(spi,0);
	voiceSpiReadWrite(spi,0);
	gpio_set_level(PIN_NUM_XDCS, 1);//关闭数据片选，SDI无效
	vTaskDelay(20 / portTICK_RATE_MS);
}

int VS_Disable_Record(spi_device_handle_t spi)
{
    VS_Write_Reg(spi,VS_MODE,0x08,0x04);
    return 0;
}

/******************************************************************
 - 功能描述：正弦测试，这是测试VS1003芯片是否正常的有效手段！！
 - 隶属模块：VS1003B模块
 - 函数属性：外部，用户可调用
 - 参数说明：x决定了正弦测试中产生的正弦波的频率，直接影响听到的
             声音的频率
 - 返回说明：无
 ******************************************************************/
void VS_sin_test(spi_device_handle_t spi,unsigned char x)
{
    VS_Write_Reg(spi,0x00,0x08,0x20);//启动测试，向0号寄存器写入0x0820   SM_SDINEW为1   SM_TEST为1

    while(!gpio_get_level(PIN_NUM_DREQ));   //等待DREQ变为高电平
    gpio_set_level(PIN_NUM_XDCS, 0);	        //打开数据片选 SDI有效
    voiceSpiReadWrite(spi,0x53);//写入以下8个字节,进入正弦测试
    voiceSpiReadWrite(spi,0xef);
    voiceSpiReadWrite(spi,0x6e);
    voiceSpiReadWrite(spi,x);   //参数x用来调整正弦测试中正弦波的频率   FsIdx (b7~b5):采样率表索引   S (b4~b0):正弦波的跃速   频率F=Fs X S / 128
    voiceSpiReadWrite(spi,0);   //比如x=126 (0b 011 11110) FsIdx=011=3   Fs=22050Hz   S=11110=30    F=22050Hz X 30 /128 =5168 Hz
    voiceSpiReadWrite(spi,0);
    voiceSpiReadWrite(spi,0);
    voiceSpiReadWrite(spi,0);
    vTaskDelay(500 / portTICK_RATE_MS);     //这里延时一段时间，为了听到“正弦音”
    voiceSpiReadWrite(spi,0x45);//写入以下8个字节，退出正弦测试
    voiceSpiReadWrite(spi,0x78);
    voiceSpiReadWrite(spi,0x69);
    voiceSpiReadWrite(spi,0x74);
    voiceSpiReadWrite(spi,0);
    voiceSpiReadWrite(spi,0);
    voiceSpiReadWrite(spi,0);
    voiceSpiReadWrite(spi,0);
    gpio_set_level(PIN_NUM_XDCS, 1);	    //关闭数据片选 ，SDI无效
}

void VS_play_record_Dat(spi_device_handle_t spi)
{
    uint16_t i=0;
    uint16_t j=0;
    // Open renamed file for reading
	ESP_LOGI(TAG, "Reading file");
	f = fopen("/sdcard/wind.mp3", "r");
	if (f == NULL) {
		ESP_LOGE(TAG, "Failed to open file for reading");
		return;
	}
	fread(music,12,1,f);
	//fgets(music, 12, f);
	fclose(f);
	f=NULL;
	// strip newline
//	char* pos = strchr(music, '\n');
//	if (pos) {
//		*pos = '\0';
//	}
//	ESP_LOGI(TAG, "Read from file: '%s'", music);

//    for(j=0;j<sizeof(music);j+=32)
//    {
//        while(!gpio_get_level(PIN_NUM_DREQ));   //等待DREQ变为高电平
//        gpio_set_level(PIN_NUM_XDCS, 0);//打开数据片选 SDI有效
//        for(i=0;i<32;i++)
//        {
//            voiceSpiReadWrite(spi,music[i+j]) ;//向vs1003传输数据
//        }
//        gpio_set_level(PIN_NUM_XDCS, 1);	        //打开数据片选 SDI无效
//        gpio_set_level(PIN_NUM_CS, 1);
//    }
}


void spi2_task(void)
{
	//Set UART1 pins(TX: IO4, RX: I05, RTS: IO18, CTS: IO19)
	uart_set_pin(UART_NUM_0, 1, 3, UART_PIN_NO_CHANGE,UART_PIN_NO_CHANGE);
	esp_err_t ret;

    spi_bus_config_t buscfg={
        .miso_io_num=PIN_NUM_MISO,
        .mosi_io_num=PIN_NUM_MOSI,
        .sclk_io_num=PIN_NUM_CLK,
        .quadwp_io_num=-1,
        .quadhd_io_num=-1
    };
    spi_device_interface_config_t devcfg={
        .clock_speed_hz=2000000,               //Clock out at 5 MHz
        .mode=0,                                //SPI mode 0
        .spics_io_num=-1,               //CS pin
        .queue_size=7,                          //We want to be able to queue 7 transactions at a time
    };

    //Initialize the SPI bus
    ret=spi_bus_initialize(HSPI_HOST, &buscfg, 1);
    assert(ret==ESP_OK);
    //Attach the LCD to the SPI bus
    ret=spi_bus_add_device(HSPI_HOST, &devcfg, &spi);
    assert(ret==ESP_OK);
    ESP_LOGI(TAG, "----------");
    //Initialize the vs1003
    VS1003_Reset(spi);
    VS_Disable_Record(spi);
    uint8_t i=0;

    while(1){
    	ESP_LOGI(TAG, "----------");
    	//VS_sin_test(spi,i++*3);

    	VS_play_record_Dat(spi);
    	//voiceSpiReadWrite(spi,0x55);
    	vTaskDelay(1000/portTICK_PERIOD_MS);
    }
}

